
/**
 * @file
 * Defines JavaScript behaviors for the uCount Barcode form.
 *
 * @see ucount_barcode.module
 */

var ucountBarcode = ucountBarcode || { 'driver': {}, 'submitReady': false };

/**
 * Focus the first textfield input element in the uCount Barcode form.
 */
ucountBarcode.autoFocus = function() {
  $('.ucount-barcode:first').focus();
};

/**
 * This method calls the current driver and allows it process
 * user input.
 *
 * @param data
 *   The input the user provided on the form.
 * @return
 *   The processed input from the driver.
 */
ucountBarcode.process = function(data) {
  if (ucountBarcode.driver.process !== undefined) {
    return ucountBarcode.driver.process(data);
  }

  return data;
};

/**
 * This method calls the current driver and allows it tell
 * the system whether or not the user has finished inputing
 * a barcode.
 *
 *
 * @param e
 *   The event handler data from the .keypress() event.
 * @param element
 *   The current element being processed.
 * @return
 *   FALSE if the barcode has not been fully input. TRUE otherwise.
 */
ucountBarcode.inputFinished = function(e, element) {
  if (ucountBarcode.driver.inputFinished !== undefined) {
    return ucountBarcode.driver.inputFinished(e, element);
  }

  return (e.which == 13);
};

/**
 * Display volunteer hours textfield if the activity field's barcode
 * translates to a volunteer activity node.
 *
 * @param activityField
 *   A selector or DOM element that reperestts the activity textfield.
 */
ucountBarcode.checkStatus = function(activityField) {
  var barcode = $(activityField).val();
  var url = Drupal.settings.basePath +'ucount/barcode/volunteer/'+ Drupal.encodeURIComponent(barcode);

  jQuery.getJSON(url, function(data, result) {
    if (result != 'success') {
      return;
    }

    if (data.field_ucount_vol_activity !== undefined && data.field_ucount_vol_activity) {
      $('#ucount-barcode-hours').slideDown('normal', function() {
        $(this).find('input:first').focus();
      });
    }
    else {
      ucountBarcode.submitReady = true;
      $('#ucount-signin').submit();
    }
  });
};

/**
 * Keypress event handler for textfield input forms on the uCount
 * Barcode form.
 *
 * @return
 *   TRUE when the data has been processed. FALSE when it has not been.
 */
ucountBarcode.keypress = function(e, element) {
  var qEle = $(element);

  if (ucountBarcode.inputFinished(e, element)) {
    var barcode = ucountBarcode.process(qEle.val());
    qEle.val(barcode);
    return true;
  }

  return false;
};

/**
 * Define behaviors for the uCount Barcode form.
 */
Drupal.behaviors.ucountBarcode = function() {
  // Attach the form submit behavior
  $('#ucount-signin:not(.ucountBarcode-processed)')
    .addClass('ucountBarcode-processed')
    .submit(function() {
      if (ucountBarcode.submitReady) {
        // Make sure we don't submit twice.
        ucountBarcode.submitReady = false;
        return true;
      }

      return false;
    });

  // Attach the activity textfield behavior.
  $('#edit-activity-0-barcode-act-wrapper')
    .find('.ucount-barcode:first')
    .addClass('ucountBarcode-processed')
    .keypress(function(e) {
      if (ucountBarcode.keypress(e, this)) {
        ucountBarcode.checkStatus(this);
      }
    });

  // Attach the regular barcode textfield behavior.
  $('.ucount-barcode:not(.ucountBarcode-processed)')
    .addClass('ucountBarcode-processed')
    .keypress(function(e) {
      if (ucountBarcode.keypress(e, this)) {
        ucountBarcode.submitReady = true;
        $('#ucount-signin').submit();
      }
    });
};

/**
 * Call autoFocus on every page load within uCount Barcode.
 */
$(document).ready(function() {
  // Clear any current values inside uCount inputs.
  $('.ucount-barcode').val('');
  
  ucountBarcode.autoFocus();
});
